
From: Vivek Goyal <vgoyal@in.ibm.com>

CPU does not save ss and esp on stack if execution was already in kernel mode
at the time of NMI occurrence.  This leads to saving of erractic values for ss
and esp.  This patch fixes the issue.  

Signed-off-by: Vivek Goyal <vgoyal@in.ibm.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/i386/kernel/crash.c |   13 +++++++++++++
 1 files changed, 13 insertions(+)

diff -puN arch/i386/kernel/crash.c~kdump-nmi-handler-segment-selector-stack arch/i386/kernel/crash.c
--- 25/arch/i386/kernel/crash.c~kdump-nmi-handler-segment-selector-stack	2005-03-28 18:03:08.000000000 -0800
+++ 25-akpm/arch/i386/kernel/crash.c	2005-03-28 18:03:08.000000000 -0800
@@ -112,7 +112,20 @@ static atomic_t waiting_for_crash_ipi;
 
 static int crash_nmi_callback(struct pt_regs *regs, int cpu)
 {
+	struct pt_regs fixed_regs;
 	local_irq_disable();
+
+	/* CPU does not save ss and esp on stack if execution is already
+	 * running in kernel mode at the time of NMI occurrence. This code
+	 * fixes it.
+	 */
+	if (!user_mode(regs)) {
+		memcpy(&fixed_regs, regs, sizeof(*regs));
+		fixed_regs.esp = (unsigned long)&(regs->esp);
+		__asm__ __volatile__("xorl %eax, %eax;");
+		__asm__ __volatile__ ("movw %%ss, %%ax;" :"=a"(fixed_regs.xss));
+		regs = &fixed_regs;
+	}
 	crash_save_this_cpu(regs, cpu);
 	disable_local_APIC();
 	atomic_dec(&waiting_for_crash_ipi);
_
